#include <math.h>

#include <qvariant.h>
#include <qlayout.h>
#include <qpainter.h>

#include <kwin/workspace.h>
#include <netwm.h>

#include <kstandarddirs.h>
#include <kapplication.h>
#include <kstddirs.h>
#include <kconfig.h>
#include <kwin/options.h>
#include <kpixmap.h>
#include <kdebug.h>

#include "sidekap.h"
#include "sidekapbuttons.h"

using namespace KWinInternal;

static uint sideMargin   = 3;
static uint cornerSize   = 15;

static bool doInit                = true;
static int toolCaptionBarHeight;
static int normalCaptionBarHeight;

extern "C"
{
  Client * allocate(Workspace * ws, WId w, int tool)
  {
    return new SideKap(ws, w, tool != 0);
  } /* allocate */
  void init()
  {
    // Nothing... Init needs Client API -> done in first client.
    doInit = true;
  } /* init */
  void reset()
  {
    // Indicates that elements are to be recalculated.
    doInit = true;
    Workspace::self()->slotResetAllClientsDelayed();
  } /* reset */
  void deinit()
  {
  } /* deinit */
}

SideKap::SideKap(Workspace * ws, WId w, bool tool, QWidget * parent, const char * name)
        : Client(ws, w, parent, name, WResizeNoErase),
          tool(tool),
          showStayOnTop(true),
          mainLayout(0),
          captionBarLayout(0),
          bottomSpacer(0),
          captionSpacer(0)
{
  setBackgroundMode(NoBackground);

  resetLayout();

  topButtonList.setAutoDelete(true);
  bottomButtonList.setAutoDelete(true);

  connect(options, SIGNAL(resetClients()), this, SLOT(slotReset()));
} // SideKap::SideKap

SideKap::~SideKap()
{
} // SideKap::~SideKap

void SideKap::resizeEvent(QResizeEvent * e)
{
  Client::resizeEvent(e);
  doShape();
  repaint();
} // SideKap::resizeEvent

void SideKap::captionChange(const QString &)
{
  doShape();
  repaint();
} // SideKap::captionChange

void SideKap::paintEvent(QPaintEvent *)
{
  int r(width());
  int b(height());
  int captionBarHeight = (tool?toolCaptionBarHeight:normalCaptionBarHeight);
  int captionBarLength = bottomSpacer->geometry().top()-5;
  QFontMetrics fm(options->font(isActive(), tool));

  QPainter p(this);

  if ((captionSpacer) && (captionSpacer->geometry().height()))
  {
    KPixmap *cp = new KPixmap();
    cp->resize(captionSpacer->geometry().height(), captionBarHeight-4);
    QPainter p2(cp);
    p2.setPen(options->color(Options::TitleBar, isActive()));
    p2.setBrush(options->color(Options::TitleBar, isActive()));
    p2.drawRect(0, 0, cp->width(), cp->height());
    p2.setFont(options->font(isActive(), tool));
    p2.setPen(options->color(Options::Font, isActive()));
    p2.drawText(0, 0, cp->width(), cp->height(), AlignVCenter, caption());
    p2.end();
    p.save();
    p.rotate(270.);
    p.translate(4-captionSpacer->geometry().bottom(), 2);
    p.drawPixmap(0, 0, *cp);
    p.flush();
    delete cp;
    p.restore();
  }

  p.setPen(options->color(Options::TitleBar, isActive()));
  p.setBrush(options->color(Options::TitleBar, isActive()));
  p.drawRect(2, 2, captionBarHeight-4, captionSpacer->geometry().top());
  QPointArray a;
  a.setPoints(4, 2, captionSpacer->geometry().bottom()-3, captionBarHeight-3, captionSpacer->geometry().bottom()-3,
                  captionBarHeight-3, captionBarLength-5, 2, captionBarLength-captionBarHeight);
  p.drawPolygon(a);

  p.setPen(colorGroup().background());
  p.setBrush(colorGroup().background());
  p.setPen(colorGroup().background().dark(120));
  p.drawLine(captionBarHeight-2, 2, r-4, 2);
  p.drawLine(r-2, 1, r-2, cornerSize-2);
  p.drawLine(captionBarHeight+sideMargin-3, b-2, r-2, b-2);
  p.drawLine(r-2, b-cornerSize+1, r-2, b-2);
  p.drawLine(captionBarHeight+sideMargin-1, b-2, captionBarHeight+sideMargin-1, b-cornerSize+1);
  p.drawLine(captionBarHeight-2, 2, captionBarHeight-2, captionBarLength-2);
  p.setPen(colorGroup().background().light());
  p.drawLine(1, 1, r-3, 1);
  p.drawLine(r-3, 2, r-3, cornerSize-2);
  p.drawLine(captionBarHeight+sideMargin-3, b-3, r-3, b-3);
  p.drawLine(r-3, b-cornerSize+1, r-3, b-3);
  p.drawLine(captionBarHeight+sideMargin-2, b-2, captionBarHeight+sideMargin-2, b-cornerSize+1);
  p.drawLine(1, 2, 1, captionBarLength-captionBarHeight-1);
  p.drawLine(1, captionBarLength-captionBarHeight, captionBarHeight-2, captionBarLength-3);

  p.setPen(Qt::black);
  p.setBrush(Qt::NoBrush);
  p.drawRect(rect());
  p.drawRect(captionBarHeight + sideMargin, 3, r-3-captionBarHeight-sideMargin, b-6);
  p.drawLine(captionBarHeight+sideMargin-3, b-1, captionBarHeight+sideMargin-3, b-cornerSize);
  p.drawPoint(captionBarHeight+sideMargin-2, b-cornerSize);
  p.drawPoint(captionBarHeight+sideMargin-1, b-cornerSize);
  p.drawPoint(r-2, b-cornerSize);
  p.drawPoint(r-3, b-cornerSize);
  p.drawPoint(r-2, cornerSize-1);
  p.drawPoint(r-3, cornerSize-1);
  p.drawLine(captionBarHeight-1, 3, captionBarHeight-1, captionBarLength-1);
  p.drawLine(captionBarHeight-1, 3, captionBarHeight+sideMargin, 3);
  p.drawLine(0, captionBarLength-captionBarHeight, captionBarHeight-1, captionBarLength-1);

  p.flush();
  p.end();
}

void SideKap::doShape()
{
  calculateElements();

  int captionBarHeight = (tool?toolCaptionBarHeight:normalCaptionBarHeight);

  QRegion mask(0, 0, width(), height());

  int r(width());
  int b(height());
  captionSpacer->changeSize(captionBarHeight, captionWidth+6, QSizePolicy::Fixed, (isShade()?QSizePolicy::Fixed:QSizePolicy::Maximum));
  captionBarLayout->invalidate();
  captionBarLayout->activate();
  mainLayout->invalidate();
  mainLayout->activate();
  int captionBarLength = bottomSpacer->geometry().top()-5;

  mask -= QRegion(0, captionBarLength-1, captionBarHeight, b-captionBarLength+1);
  if (isShade())
    mask -= QRegion(captionBarHeight, 0, r-captionBarHeight, b);
  else
  {
    mask -= QRegion(captionBarHeight, 4, sideMargin, b-cornerSize-4);
    mask -= QRegion(captionBarHeight, b - cornerSize, sideMargin-3, cornerSize);
    mask -= QRegion(r-3, cornerSize, 3, b-2*cornerSize);
  }
  QPointArray a;
  a.setPoints(3, 0, captionBarLength-captionBarHeight, captionBarHeight-1, captionBarLength-1, 0, captionBarLength-1);
  mask -= QRegion(a);

  setMask(mask);
} // SideKap::doShape

void SideKap::showEvent(QShowEvent *)
{
  doShape();
  repaint();
} // SideKap::showEvent

void SideKap::windowWrapperShowEvent(QShowEvent *)
{
  doShape();
  repaint();
} // SideKap::windowWrapperShowEvent

void SideKap::mouseDoubleClickEvent(QMouseEvent * e)
{
  if (captionSpacer->geometry().contains(e->pos()))
    workspace()->performWindowOperation(this, options->operationTitlebarDblClick());
} // SideKap::mouseDoubleClickEvent

void SideKap::stickyChange(bool b)
{
  emit(stkyChange(b));
} // SideKap::stickyChange

void SideKap::maximizeChange(bool b)
{
  emit(maxChange(b));
} // SideKap::maximizeChange

void SideKap::activeChange(bool)
{
  doShape();
  repaint();
} // SideKap::activeChange

Client::MousePosition SideKap::mousePosition(const QPoint & p) const
{
  int x = p.x();
  int y = p.y();
  int captionBarHeight = (tool?toolCaptionBarHeight:normalCaptionBarHeight);

  if (y > height()-cornerSize)
    if (x < captionBarHeight+sideMargin+1+cornerSize)
      return Client::BottomLeft;
    else if (x > width()-cornerSize)
      return Client::BottomRight;
    else
      return Client::Bottom;
  if (y < 4)
    if (x < cornerSize)
      return Client::TopLeft;
    else if (x > width()-cornerSize)
      return Client::TopRight;
    else
      return Client::Top;
      
  if ((x < 4) && (y < cornerSize))
    return Client::TopLeft;
  if ((x > width()-cornerSize) && (y < cornerSize))
    return Client::TopRight;

  return Client::Center;

  return Client::mousePosition(p);
} // SideKap::mousePosition

void SideKap::slotReset()
{
  resetLayout();
  repaint();
} // SideKap::slotReset

void SideKap::slotMaximize(int button)
{
  switch (button)
  {
    case MidButton:
      maximize(MaximizeVertical);
      break;

    case RightButton:
      maximize(MaximizeHorizontal);
      break;

    case LeftButton:
    default:
      maximize(MaximizeFull);
  }
} // SideKap::slotMaximize

void SideKap::toggleStayOnTop()
{
  setStaysOnTop(!staysOnTop());
} // SideKap::toggleStayOnTop

SideKapButton * SideKap::createButton(const QString & s, QWidget * parent)
{
  SideKapButton * b = 0;

  if (("Help" == s) && providesContextHelp())
  {
    b = new SideKapButtonHelp(parent, "Help");
    connect(b, SIGNAL(help()), this, SLOT(contextHelp()));
  }

  else if ("Sticky" == s)
  {
    b = new SideKapButtonSticky(isSticky(), parent, "Sticky");
    connect(b, SIGNAL(toggleSticky()), this, SLOT(toggleSticky()));
    connect(this, SIGNAL(stkyChange(bool)), b, SLOT(slotStickyChange(bool)));
  }

  else if ("StayOnTop" == s)
  {
    b = new SideKapButtonStaysOnTop(parent, "Stay on top");
    connect(b, SIGNAL(toggleStaysOnTop()), this, SLOT(toggleStayOnTop()));
  }

  else if ("Iconify" == s && isMinimizable())
  {
    b = new SideKapButtonIconify(parent, "Minimize");
    connect(b, SIGNAL(iconify()), this, SLOT(iconify()));
  }

  else if ("Maximize" == s && isMaximizable())
  {
    b = new SideKapButtonMaximize(isMaximized(), parent, "Maximize");
    connect(b, SIGNAL(maximize(int)), this, SLOT(slotMaximize(int)));
    connect(this, SIGNAL(maxChange(bool)), b, SLOT(slotMaximizeChange(bool)));
  }

  else if ("Close" == s)
  {
    b = new SideKapButtonClose(parent, "Close");
    connect(b, SIGNAL(closeWindow()), this, SLOT(closeWindow()));
  }

  else if ("Lower" == s)
  {
    b = new SideKapButtonLower(parent, "Lower");
    connect(b, SIGNAL(lowerWindow()), this, SLOT(slotLowerWindow()));
  }

  return b;
} // SideKap::createButton

void SideKap::createButtons()
{
  topButtonList.clear();
  bottomButtonList.clear();

  QString buttons = options->titleButtonsLeft() + "|T" + options->titleButtonsRight();
  QPtrList<SideKapButton> *buttonList = &topButtonList;
  for (unsigned int i = 0; i < buttons.length(); ++i)
  {
    SideKapButton * tb = 0;
    switch (buttons[i].latin1())
    {
      case 'S': // Sticky
        tb = createButton("Sticky", this);
        break;

      case 'H': // Help
        tb = createButton("Help", this);
        break;

      case 'I': // Minimize
        tb = createButton("Iconify", this);
        break;

      case 'A': // Maximize
        tb = createButton("Maximize", this);
        break;

      case 'X': // Close
        tb = createButton("Close", this);
        break;

      case 'T': // Stay On Top
        tb = createButton("StayOnTop", this);
        break;

      case '|':
        buttonList = &bottomButtonList;
        break;
    }
    if (0 != tb)
      buttonList->append(tb);
  }
} // SideKap::createButtons

void SideKap::calculateElements()
{
  doInit = false;

  QFontMetrics fm(options->font(isActive(), false));
  normalCaptionBarHeight = 6+fm.height();
  QFontMetrics fm1(options->font(isActive(), true));
  toolCaptionBarHeight = 6+fm1.height();
  QFontMetrics fm2(options->font(isActive(), tool));
  captionWidth = fm2.width(caption());
} // SideKap::calculateElements

void SideKap::resetLayout()
{
  calculateElements();

  int buttonWidth = 12;
  int captionBarHeight = (tool?toolCaptionBarHeight:normalCaptionBarHeight);

  captionSpacer = new QSpacerItem(captionBarHeight, captionWidth, QSizePolicy::Fixed, (isShade()?QSizePolicy::Fixed:QSizePolicy::Maximum));
  bottomSpacer = new QSpacerItem(captionBarHeight, 0, QSizePolicy::Fixed, QSizePolicy::Expanding);

  delete mainLayout;
  mainLayout = new QVBoxLayout(this, 0, 0);
  mainLayout->addSpacing(4);

  QHBoxLayout * topLayout = new QHBoxLayout(mainLayout, 0, 0);

  captionBarLayout = new QVBoxLayout(topLayout, 0, 0);
  createButtons();
  captionBarLayout->setSpacing(3);
  for (QPtrListIterator<SideKapButton> it(topButtonList); it.current(); ++it)
  {
    captionBarLayout->addWidget(it.current(), 0, Qt::AlignHCenter);
    captionBarLayout->setStretchFactor(it.current(), 0);
    it.current()->setFixedSize(buttonWidth, buttonWidth);
  }
  captionBarLayout->addItem(captionSpacer);
  for (QPtrListIterator<SideKapButton> it(bottomButtonList); it.current(); ++it)
  {
    captionBarLayout->addWidget(it.current(), 0, Qt::AlignHCenter);
    it.current()->setFixedSize(buttonWidth, buttonWidth);
  }
  captionBarLayout->addSpacing(captionBarHeight);
  captionBarLayout->addItem(bottomSpacer);

  topLayout->addSpacing(1+sideMargin);
  topLayout->addWidget(windowWrapper());
  topLayout->addSpacing(4);

  mainLayout->addSpacing(4);
} // SideKap::resetLayout

void SideKap::animateIconifyOrDeiconify(bool /* iconify */)
{
} // SideKap::animateIconifyOrDeiconify
