#include <qpainter.h>

#include "sidekapbuttons.h"
#include <kpixmap.h>

using namespace KWinInternal;

SideKapButton::SideKapButton(QWidget * parent, const QString& tip)
  : KWinWidgetButton(parent, 0, 0, tip),
    mouseOver  (false),
    mouseDown  (false)
{
  setSizePolicy(QSizePolicy(QSizePolicy::Minimum, QSizePolicy::Minimum));
  setBackgroundMode(NoBackground);
} // SideKapButton::SideKapButton

SideKapButton::~SideKapButton()
{
  // Empty.
} // SideKapButton::~SideKapButton

void SideKapButton::mousePressEvent(QMouseEvent *e)
{
  KWinWidgetButton::mousePressEvent(e);
  mouseDown = true;
  repaint();
} // SideKapButton::mousePressEvent

void SideKapButton::mouseReleaseEvent(QMouseEvent * e)
{
  mouseDown = false;
  repaint();

  if (rect().contains(e->pos()))
  {
    clickEvent(e->button());
  }
  KWinWidgetButton::mouseReleaseEvent(e);
} // SideKapButton::mouseReleaseEvent

void SideKapButton::enterEvent(QEvent *e)
{
  KWinWidgetButton::enterEvent(e);
  mouseOver = true;
  repaint();
} // SideKapButton::enterEvent

void SideKapButton::leaveEvent(QEvent *e)
{
  KWinWidgetButton::leaveEvent(e);
  mouseOver = false;
  repaint();
} // SideKapButton::leaveEvent

void SideKapButton::paintEvent(QPaintEvent *)
{
  QBitmap mp(width(), height(), true);
  QPainter mpp(&mp);
  mpp.setPen(Qt::color1);
  mpp.setBrush(Qt::color1);
  mpp.drawEllipse(0, 0, width(), height());
  setMask(mp);

  QPen* highlightPen = 0;

  if (mouseDown)
    highlightPen = new QPen(colorGroup().light());

  else if (mouseOver)
    highlightPen = new QPen(colorGroup().highlight());

  QPainter p(this);

  if (highlightPen)
  {
    p.setPen(*highlightPen);
    p.setBrush(colorGroup().background());
    p.drawEllipse(0, 0, width(), height());
//    p.drawArc(1, 1, width()-2, height()-2, 0, 5760);
  }
  else
  {
    p.setBrush(colorGroup().background());
    p.drawEllipse(0, 0, width(), height());
    p.setPen(colorGroup().background().light());
    p.drawArc(0, 0, width(), height(), 2880, 5760);
    p.setPen(colorGroup().background().dark());
    p.drawArc(0, 0, width(), height(), 0, 2880);
  }
  delete highlightPen;

  // Draw icon.

  QPoint center(rect().center());

  int bwby2(bitmap.width() / 2);
  int bhby2(bitmap.height() / 2);

  p.setBrush(NoBrush);
  p.setPen(Qt::black);

  p.drawPixmap(center.x() - bwby2 + 1, center.y() - bhby2 + 1, bitmap);
} // SideKapButton::paintEvent

QSize SideKapButton::sizeHint() const
{
  return QSize(16, 16);
} // SideKapButton::sizeHint

QSize SideKapButton::minimumSizeHint() const
{
  return QSize(14, 14);
} // SideKapButton::minimumSizeHint

void SideKapButton::setBitmap(const QBitmap & b)
{
  bitmap = b;
  repaint();
}

void SideKapButton::resizeEvent(QResizeEvent *)
{
  repaint();
} // SideKapButton::resizeEvent

static unsigned char close_bits[] =
{
   0x42, 0xe7, 0x7e, 0x3c, 0x3c, 0x7e, 0xe7, 0x42
};

SideKapButtonClose::SideKapButtonClose(QWidget * parent, const QString& tip)
  : SideKapButton(parent, tip)
{
  QBitmap b(8, 8, close_bits, true);
  b.setMask(b);
  setBitmap(b);
} // SideKapButtonClose::SideKapButtonClose

void SideKapButtonClose::clickEvent(int)
{
  emit(closeWindow());
} // SideKapButtonClose::clickEvent

static unsigned char help_bits[] =
{
   0x18, 0x18, 0x00, 0x1c, 0x18, 0x18, 0x18, 0x3c
};

SideKapButtonHelp::SideKapButtonHelp(QWidget * parent, const QString& tip)
  : SideKapButton(parent, tip)
{
  QBitmap b(8, 8, help_bits, true);
  b.setMask(b);
  setBitmap(b);
} // SideKapButtonHelp::SideKapButtonHelp

void SideKapButtonHelp::clickEvent(int)
{
  emit(help());
} // SideKapButtonHelp::clickEvent

static unsigned char iconify_bits[] =
{
   0x00, 0x00, 0x00, 0x7e, 0x7e, 0x3c, 0x18, 0x00
};

SideKapButtonIconify::SideKapButtonIconify(QWidget * parent, const QString& tip)
  : SideKapButton(parent, tip)
{
  QBitmap b(8, 8, iconify_bits, true);
  b.setMask(b);
  setBitmap(b);
} // SideKapButtonIconify::SideKapButtonIconify

void SideKapButtonIconify::clickEvent(int)
{
  emit(iconify());
} // SideKapButtonIconify::clickEvent

static unsigned char lower_bits[] =
{
   0x1f, 0x1f, 0x1f, 0xff, 0x8f, 0x88, 0x88, 0xf8
};

SideKapButtonLower::SideKapButtonLower(QWidget * parent, const QString& tip)
  : SideKapButton(parent, tip)
{
  QBitmap b(8, 8, lower_bits, true);
  b.setMask(b);
  setBitmap(b);
} // SideKapButtonLower::SideKapButtonLower

void SideKapButtonLower::clickEvent(int)
{
  emit(lowerWindow());
} // SideKapButtonLower::clickEvent

static unsigned char maximize_bits[] =
{
   0xfc, 0xf8, 0xf0, 0xe0, 0xc0, 0x80, 0x00, 0x00
};

static unsigned char unmaximize_bits[] =
{
   0x00, 0x00, 0x01, 0x03, 0x07, 0x0f, 0x1f, 0x3f
};

SideKapButtonMaximize::SideKapButtonMaximize(bool max, QWidget * parent, const QString& tip)
  : SideKapButton(parent, tip)
{
  QBitmap b(8, 8, max ? unmaximize_bits : maximize_bits, true);
  b.setMask(b);
  setBitmap(b);
} // SideKapButtonMaximize::SideKapButtonMaximize

void SideKapButtonMaximize::slotMaximizeChange(bool max)
{
  QBitmap b(8, 8, max ? unmaximize_bits : maximize_bits, true);
  b.setMask(b);
  setBitmap(b);
} // SideKapButtonMaximize::slotMaximizeChange

void SideKapButtonMaximize::clickEvent(int button)
{
  emit(maximize(button));
} // SideKapButtonMaximize::clickEvent

static unsigned char sticky_bits[] =
{
   0x20, 0x70, 0xfa, 0x7e, 0x3c, 0x1c, 0x32, 0x01
};

static unsigned char unsticky_bits[] =
{
   0x1c, 0x1c, 0x1c, 0x3e, 0x7f, 0x08, 0x08, 0x08
};

SideKapButtonSticky::SideKapButtonSticky(bool sticky, QWidget * parent, const QString& tip)
  : SideKapButton(parent, tip)
{
  QBitmap b(8, 8, sticky ? unsticky_bits : sticky_bits, true);
  b.setMask(b);
  setBitmap(b);
} // SideKapButtonSticky::SideKapButtonSticky

void SideKapButtonSticky::slotStickyChange(bool sticky)
{
  QBitmap b(8, 8, sticky ? unsticky_bits : sticky_bits, true);
  b.setMask(b);
  setBitmap(b);
} // SideKapButtonSticky::slotStickyChange

void SideKapButtonSticky::clickEvent(int)
{
  emit(toggleSticky());
} // SideKapButtonSticky::clickEvent

static unsigned char stayontop_bits[] =
{
   0xf8, 0x88, 0x8f, 0x8f, 0xff, 0x3f, 0x3f, 0x3f
};

SideKapButtonStaysOnTop::SideKapButtonStaysOnTop(QWidget * parent, const QString& tip)
  : SideKapButton(parent, tip)
{
  QBitmap b(8, 8, stayontop_bits, true);
  b.setMask(b);
  setBitmap(b);
} // SideKapButtonStaysOnTop::SideKapButtonStaysOnTop

void SideKapButtonStaysOnTop::clickEvent(int)
{
  emit(toggleStaysOnTop());
} // SideKapButtonStaysOnTop::clickEvent
